"""Exports for the Pypechain package."""

# This tool is meant to be used as a command line tool.  It is exported here for testing.  Do not
# rely on this being available in the future.
from .main import main as pypechain_cli
